#include <stdlib.h>
#include <signal.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <dtk.h>
#include <dtkX11.h>
#include <dgl.h>
#include <dglX11.h>
#include <math.h>

#define SPHERE 4
#ifndef PI
#define PI 3.141592654
#endif

struct XYZ
{
  float x;
  float y;
  float z;
};

bool go;
GLdouble x=0, y=0, r=0.3, x2=0, y2=0;
GLuint i=0, i2=0, step=0;

static float material_red[] = {1.f, 0.f, 0.f, 1.f};

void signal_catcher(int sig)
{
  go = false;
}
  
void CreateSphere(XYZ c,double r,int n)
{
   int i,j;
   double theta1,theta2,theta3;
   XYZ e,p;

   glNewList(SPHERE, GL_COMPILE);
   if (r < 0)
      r = -r;
   if (n < 0)
      n = -n;
   if (n < 4 || r <= 0) 
     {
      glBegin(GL_POINTS);
      glVertex3f(c.x,c.y,c.z);
      glEnd();
      return;
     }

   for (j=0;j<n/2;j++) 
     {
       theta1 = j *2*PI / n - (PI/2);
       theta2 = (j + 1) * 2*PI / n - (PI/2);
       
       glBegin(GL_QUAD_STRIP);
       for (i=0;i<=n;i++) {
         theta3 = i * 2*PI / n;
	 
         e.x = cos(theta2) * cos(theta3);
         e.y = sin(theta2);
         e.z = cos(theta2) * sin(theta3);
         p.x = c.x + r * e.x;
         p.y = c.y + r * e.y;
         p.z = c.z + r * e.z;

         glNormal3f(e.x,e.y,e.z);
         glTexCoord2f(i/(double)n,2*(j+1)/(double)n);
         glVertex3f(p.x,p.y,p.z);

         e.x = cos(theta1) * cos(theta3);
         e.y = sin(theta1);
         e.z = cos(theta1) * sin(theta3);
         p.x = c.x + r * e.x;
         p.y = c.y + r * e.y;
         p.z = c.z + r * e.z;
	 
         glNormal3f(e.x,e.y,e.z);
         glTexCoord2f(i/(double)n,2*j/(double)n);
         glVertex3f(p.x,p.y,p.z);
       }
       glEnd();
     }
   
   glEndList();
}



void gl_init_callback(const dglDisplayInfo *info, void *data)
{
  unsigned int index = info->thread->threadNumber();

  XYZ a;
  
  a.x=0.0;
  a.y=0.0;
  a.z=0.0;

  CreateSphere(a, r, 20);

  glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_NORMALIZE);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glShadeModel(GL_SMOOTH);
}


void display_callback(const dglDisplayInfo *info, void *data)
{
 
  static float light_ambient[]   = {.5f, .5f, .5f, 1.f};
  static float light_diffuse[]   = {.5f, .5f, .5f, 1.f};
  static float light_direction[] = {0.f, -1.f, 1.f, 0.f};

  unsigned int index = info->thread->threadNumber();


  glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, light_direction);

  glPushMatrix();
  glNormal3f(0.f, -1.f, 0.f);
  glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, material_red);
  glTranslatef(0.f, 1.0, 0.f);
  glCallList(SPHERE);
  glPopMatrix();

}

int main(int argc, char **argv)
{
  signal(SIGINT, signal_catcher);
  signal(SIGTERM, signal_catcher);
  go = true;

  dglManager *manager = new dglManager();

  if (!manager || manager->isInvalid()) {
    printf("Could not create a valid manager.\n");
    return EXIT_FAILURE;
  }

  manager->initGLCallback(gl_init_callback, NULL);
  manager->displayCallback(display_callback, NULL);
  manager->config();
  
  dglDisplay *display = manager->display(0);
  if (!display || display->isInvalid()) {
    printf("Could not find a valid display object.\n");
    return EXIT_FAILURE;
  }
  display->clearBuffers(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  display->coordSystem(dglDisplay::NAVIGATED_NDC_COORDS);

  while (go && (manager->state & DTK_ISRUNNING)) {
    manager->sync();
    manager->frame();
  }

  delete manager;

  return EXIT_SUCCESS;
}
